@@ -1,4 +1,4 @@
-# Copyright (c) 2016 - 2021, Adrian Dusa
+# Copyright (c) 2016 - 2022, Adrian Dusa
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
@@ -23,12 +23,18 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-`setColnames` <- function(matrix, colnames) {
+`setColnames` <- function(
+    matrix, colnames
+) {
     invisible(.Call("C_setColnames", matrix, colnames))
 }
-`setRownames` <- function(matrix, rownames) {
+`setRownames` <- function(
+    matrix, rownames
+) {
     invisible(.Call("C_setRownames", matrix, rownames))
 }
-`setDimnames` <- function(matrix, nameslist) {
+`setDimnames` <- function(
+    matrix, nameslist
+) {
     invisible(.Call("C_setDimnames", matrix, nameslist))
 }
